<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00010000ParametrizacaoGeralFactory
{

    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function inserirParametrizacao($params)
    {
        $sql = "INSERT INTO FCM_PARAMGERAL (NRPARAMGERAL,  NRORG, IDATIVO, CDOPERINCLUSAO, NRORGINCLUSAO, IDVALIDACRIATAREFA, IDUTILIZAGEOLOCAL, IDUTILIZAGEOLOCALINITAR, IDHABILITAOFFLINE, 
                            IDTIPOCARREGAOFFLINE, QTDIASCARREGAOFFLINE, DTINICARREGAOFFLINE, DTFIMCARREGAOFFLINE, IDNOTIFNAOCONFORM, LISTNOTIFNAOCONFORM, EMAILSNOTIFNAOCONFROM, IDUTILIZAHABILIDADE)
                     VALUES (:NRPARAMGERAL, :NRORG,  :IDATIVO, :CDOPERINCLUSAO, :NRORGINCLUSAO, :IDVALIDACRIATAREFA, :IDUTILIZAGEOLOCAL, :IDUTILIZAGEOLOCALINITAR, :IDHABILITAOFFLINE, 
                             :IDTIPOCARREGAOFFLINE, :QTDIASCARREGAOFFLINE, :DTINICARREGAOFFLINE, :DTFIMCARREGAOFFLINE, :IDNOTIFNAOCONFORM, :LISTNOTIFNAOCONFORM, :EMAILSNOTIFNAOCONFROM, :IDUTILIZAHABILIDADE)";
        
        $this->connection->executeStatement($sql, $params);
    }

    public function updateParametrizacao($params)
    {
        
        $sql = "UPDATE FCM_PARAMGERAL 
                  SET CDOPERULTATU        = :CDOPERULTATU,
                  IDVALIDACRIATAREFA      = :IDVALIDACRIATAREFA,
                  IDUTILIZAGEOLOCAL       = :IDUTILIZAGEOLOCAL,
                  IDUTILIZAGEOLOCALINITAR = :IDUTILIZAGEOLOCALINITAR,
                  IDHABILITAOFFLINE       = :IDHABILITAOFFLINE,
                  DTINICARREGAOFFLINE     = :DTINICARREGAOFFLINE,
                  DTFIMCARREGAOFFLINE     = :DTFIMCARREGAOFFLINE,
                  QTDIASCARREGAOFFLINE    = :QTDIASCARREGAOFFLINE,
                  NRPARAMGERAL            = :NRPARAMGERAL,
                  IDTIPOCARREGAOFFLINE    = :IDTIPOCARREGAOFFLINE,
                  IDATIVO                 = :IDATIVO,
                  NRORGULTATU             = :NRORGULTATU,
                  DTULTATU                = SYSDATE,
                  IDNOTIFNAOCONFORM       = :IDNOTIFNAOCONFORM,
                  LISTNOTIFNAOCONFORM     = :LISTNOTIFNAOCONFORM,
                  EMAILSNOTIFNAOCONFROM   = :EMAILSNOTIFNAOCONFROM,
                  IDUTILIZAHABILIDADE     = :IDUTILIZAHABILIDADE
                WHERE NRPARAMGERAL = :NRPARAMGERAL";
        
        $this->connection->executeStatement($sql, $params);
    }

    public function getParametrizacoes()
    {
        $sql = "SELECT NRPARAMGERAL, IDVALIDACRIATAREFA, IDUTILIZAGEOLOCAL, IDUTILIZAGEOLOCALINITAR, IDHABILITAOFFLINE,
                       IDTIPOCARREGAOFFLINE, QTDIASCARREGAOFFLINE, TO_CHAR(DTINICARREGAOFFLINE, 'DD/MM/YY') DTINICAL,
                       IDNOTIFNAOCONFORM, LISTNOTIFNAOCONFORM, EMAILSNOTIFNAOCONFROM,
                       TO_CHAR(DTFIMCARREGAOFFLINE, 'DD/MM/YY') DTFINAL, IDUTILIZAHABILIDADE
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getMascara()
    {

        $sql = "SELECT IDOBRIGACDLOCAT, IDUTILIZAMASCCDLOCAT, DSMASCCDLOCAT, IDALTERAMASCCDLOCAT 
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function updateMascara($params)
    {
        $sql = "UPDATE FCM_PARAMGERAL 
                SET IDOBRIGACDLOCAT = :IDOBRIGACDLOCAT,
                IDUTILIZAMASCCDLOCAT = :IDUTILIZAMASCCDLOCAT,
                DSMASCCDLOCAT = : DSMASCCDLOCAT,
                IDALTERAMASCCDLOCAT = : IDALTERAMASCCDLOCAT,
                NRORGINCLUSAO = :NRORGINCLUSAO,
                DTULTATU = SYSDATE,
                CDOPERULTATU = :CDOPERULTATU,
                NRORGULTATU = :NRORGULTATU";

        $this->connection->executeStatement($sql, $params);
    }

    public function getCountMascara()
    {

        $sql = "SELECT COUNT(NRPARAMGERAL) AS QTA 
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    // INTEGRAÇÃO COM SM
    public function validarItem($parametros)
    {

        $sql = "SELECT COUNT(*) CONT
                    FROM FCM_RELSERVITEMTAR
                    WHERE CDSERVORCA = :CDSERVORCA
                    AND CDTPMEDIDASERV = :CDTPMEDIDASERV
                    AND NRATIVIDADE = :NRATIVIDADE
                    AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0]['CONT'];
    }

    public function inserirRelacionamento($parametros)
    {

        $sql = "INSERT INTO FCM_RELSERVITEMTAR(NRRELSERVITEMTAR, CDSERVORCA, CDTPMEDIDASERV, NRATIVIDADE, IDATIVO, 
                                               NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRRELSERVITEMTAR, :CDSERVORCA, :CDTPMEDIDASERV, :NRATIVIDADE, 'S' , :NRORG, SYSDATE, 
                        :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getRelacionamentoServicoTipoMedida()
    {

        $sql = "SELECT  ROWNUM AS ID, RELACIONAMENTO.* 
                    FROM (SELECT DISTINCT FCM_RELSERVITEMTAR.CDSERVORCA, FCT_SERVICOORCA.NMSERVORCA, FCM_RELSERVITEMTAR.CDTPMEDIDASERV, FCT_TIPOMEDIDASERV.DSTPMEDIDASERV
                    FROM FCM_RELSERVITEMTAR
                    JOIN FCT_TIPOMEDIDASERV
                        ON FCT_TIPOMEDIDASERV.CDTPMEDIDASERV = FCM_RELSERVITEMTAR.CDTPMEDIDASERV
                    JOIN FCT_SERVICOORCA
                        ON FCT_SERVICOORCA.CDSERVORCA = FCM_RELSERVITEMTAR.CDSERVORCA
                        WHERE FCM_RELSERVITEMTAR.IDATIVO = 'S') RELACIONAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getItensRelacionamentoServicoTipoMedida($parametros)
    {

        $sql = "SELECT FCM_RELSERVITEMTAR.NRRELSERVITEMTAR, FCM_ATIVIDADE.NRATIVIDADE, FCM_ATIVIDADE.NMATIVIDADE 
                        FROM FCM_RELSERVITEMTAR
                        JOIN FCM_ATIVIDADE 
                          ON FCM_ATIVIDADE.NRATIVIDADE = FCM_RELSERVITEMTAR.NRATIVIDADE
                       WHERE FCM_RELSERVITEMTAR.CDTPMEDIDASERV = :CDTPMEDIDASERV
                         AND FCM_RELSERVITEMTAR.CDSERVORCA = :CDSERVORCA
                         AND FCM_RELSERVITEMTAR.IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function inativarServicos($parametros)
    {

        $sql = "UPDATE FCM_RELSERVITEMTAR
                    SET IDATIVO = 'N',
                        DTULTATU = SYSDATE,
                        CDOPERULTATU =:CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU
                    WHERE CDSERVORCA = :CDSERVORCA
                    AND CDTPMEDIDASERV = :CDTPMEDIDASERV";

        $this->connection->executeStatement($sql, $parametros);
    }

    // ------- PARAMETRIZAÇÃO GEOLOCALIZAÇÃO ----------
    public function getGeolocalizacao()
    {
        $sql = "SELECT IDUTILIZAGEOLOCAL, IDUTILIZAGEOLOCALINITAR
        FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
}
