<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00040000ModeloAuditoriaFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getTarefasAuditoria()
    {
        $sql = "SELECT FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_AUDITORIA.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_MODELOAUDIT.NRMODELOAUDIT, 
                        FCM_MODELOAUDIT.NMMODELOAUDIT, FCM_AUDITORIA.NRSEQCONTRATO, FCT_CONTRATO.NRCONTRATOFCT, FCM_AUDITORIA.CDOPERADOR, OPERADOR.NMOPERADOR, FCM_AUDITORIA.IDSITUACAOAUDIT, 
                        TO_CHAR(FCM_AUDITORIA.DTHORAAUDIT, 'DD/MM/YYYY') DTHORAAUDIT, FCM_AUDITORIA.IDTIPORECORRENCIA, FCM_TIPOLOCALCLI.NMTIPOLOCALCLI, CLIENTE.NMRAZSOCCLIE, CLIENTE.NMFANTCLIE
                    FROM FCM_AUDITORIA
                    LEFT JOIN FCM_LOCALCLI
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_AUDITORIA.NRLOCALCLIENTE
                    JOIN CLIENTE
                    ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                    JOIN FCM_TIPOLOCALCLI
                    ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                    LEFT JOIN FCM_MODELOAUDIT
                    ON FCM_MODELOAUDIT.NRMODELOAUDIT = FCM_AUDITORIA.NRMODELOAUDIT
                    LEFT JOIN FCT_CONTRATO
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCM_AUDITORIA.NRSEQCONTRATO
                    LEFT JOIN OPERADOR
                    ON OPERADOR.CDOPERADOR = FCM_AUDITORIA.CDOPERADOR";
            
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteTopicosExecucao($auditorias)
    {
        $sql = "DELETE FROM FCM_RESULTADOAUDIT
                    WHERE NRAUDITORIA IN ($auditorias)";

        $this->connection->executeStatement($sql);
    }

    public function getNrResultados($auditorias)
    {
        $sql = "SELECT NRRESULTADOAUDIT FROM FCM_RESULTADOAUDIT WHERE NRAUDITORIA IN ($auditorias)";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function deleteCriteriosExecucao($resultados)
    {
        $sql = "DELETE FROM FCM_RELACRESULTCRIT
                    WHERE NRRESULTADOAUDIT IN ($resultados)";

        $this->connection->executeStatement($sql);
    }

    public function inserirTopicosExecucao($parametros)
    {
        $sql = "INSERT INTO FCM_RESULTADOAUDIT (NRRESULTADOAUDIT, NRAUDITORIA, NRMODELOAUDIT, NRTOPICOAUDIT, NRCATEGORIAAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRRESULTADOAUDIT, :NRAUDITORIA, :NRMODELOAUDIT, :NRTOPICOAUDIT, :NRCATEGORIAAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirCriteriosExecucao($parametros)
    {
        $sql = "INSERT INTO FCM_RELACRESULTCRIT (NRRELACRESULTCRIT, NRRESULTADOAUDIT, NRCRITERIOAUDIT, NRTOPICOAUDIT, IDREALIZADO, IDATIVO, DTINCLUSAO, NRORG, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRRELACRESULTCRIT, :NRRESULTADOAUDIT, :NRCRITERIOAUDIT, :NRTOPICOAUDIT, 'N', 'S', SYSDATE, :NRORG, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteAuditoria($auditorias)
    {
        $sql = "DELETE FROM FCM_AUDITORIA
                    WHERE NRAUDITORIA IN ($auditorias)";

        $this->connection->executeStatement($sql);
    }

    public function getCriteriosModelo($parametros)
    {
        $sql = "SELECT NRCRITERIOAUDIT
                    FROM FCM_RELACCATEGCRIT
                    JOIN FCM_RELACMODCATEG
                    ON FCM_RELACMODCATEG.NRCATEGORIAAUDIT = FCM_RELACCATEGCRIT.NRCATEGORIAAUDIT
                    WHERE FCM_RELACCATEGCRIT.NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT";
                
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getTopicosModelo($parametros)
    {
        $sql = "SELECT FCM_RELACCATTOPIC.NRCATEGORIAAUDIT, FCM_RELACCATTOPIC.NRTOPICOAUDIT
                    FROM FCM_RELACCATTOPIC
                    JOIN FCM_RELACMODCATEG
                    ON FCM_RELACMODCATEG.NRCATEGORIAAUDIT = FCM_RELACCATTOPIC.NRCATEGORIAAUDIT
                    WHERE FCM_RELACMODCATEG.NRMODELOAUDIT = :NRMODELOAUDIT";
                
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function insertTarefaAuditoria($parametros)
    {
        $sql = "INSERT INTO FCM_AUDITORIA (NRAUDITORIA, NMAUDITORIA, NRLOCALCLIENTE, NRMODELOAUDIT, NRSEQCONTRATO, CDOPERADOR, IDSITUACAOAUDIT, DTHORAAUDIT, IDATIVO,
                    NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, IDTIPORECORRENCIA, IDTIPOMARCACAO)
                    VALUES (:NRAUDITORIA, :NMAUDITORIA, :NRLOCALCLIENTE, :NRMODELOAUDIT, :NRSEQCONTRATO, :CDOPERADOR, '1', :DTHORAAUDIT, 'S',
                    :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :IDTIPORECORRENCIA, :IDTIPOMARCACAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    // Funções Tab Modelo
    public function deleteModelo($parametros)
    {
        $sql = "DELETE FROM FCM_MODELOAUDIT
                    WHERE NRMODELOAUDIT = :NRMODELOAUDIT";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteRelacoesModelo($categorias, $modelo)
    {

        $sql = "DELETE FROM FCM_RELACCATTOPIC
                                WHERE NRCATEGORIAAUDIT IN ($categorias)
                                AND NRMODELOAUDIT = $modelo";
        
        $this->connection->executeStatement($sql);
                                
        $sql2 = "DELETE FROM FCM_RELACCATEGCRIT
                    WHERE NRCATEGORIAAUDIT IN ($categorias)
                    AND NRMODELOAUDIT = $modelo";

        $this->connection->executeStatement($sql2);

        $sql3 = "DELETE FROM FCM_RELACMODCATEG
                                WHERE NRCATEGORIAAUDIT IN ($categorias)
                                AND NRMODELOAUDIT = $modelo";
        
        $this->connection->executeStatement($sql3);
    }

    public function validaModeloAuditoria($parametros)
    {
        $sql = "SELECT COUNT (NRMODELOAUDIT) AS CONT
                    FROM FCM_AUDITORIA
                    WHERE NRMODELOAUDIT = :NRMODELOAUDIT";
        
        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0];
    }

    public function getCategoriaPorModelo($parametros)
    {
        $sql = "SELECT NRCATEGORIAAUDIT
                    FROM FCM_RELACMODCATEG
                    WHERE NRMODELOAUDIT = :NRMODELOAUDIT";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function deleteRelacaoCatModelo($categorias, $modelo)
    {
        $sql = "DELETE FROM FCM_RELACMODCATEG
                    WHERE NRCATEGORIAAUDIT IN ($categorias)
                    AND NRMODELOAUDIT = $modelo";

        $this->connection->executeStatement($sql);
    }

    public function deleteRelacaoCatCriterios($categorias, $modelo)
    {
        $sql = "DELETE FROM FCM_RELACCATEGCRIT
                    WHERE NRCATEGORIAAUDIT IN ($categorias)
                    AND NRMODELOAUDIT = $modelo";

        $this->connection->executeStatement($sql);
    }

    public function deleteRelacaoCatTopicos($topicos, $modelo)
    {
        $sql = "DELETE FROM FCM_RELACCATTOPIC
                    WHERE NRTOPICOAUDIT IN ($topicos)
                    AND NRMODELOAUDIT = $modelo";

        $this->connection->executeStatement($sql);
    }

    public function getEstruturaModelo($parametros)
    {
        $sql = "SELECT 'C' || FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT AS CODIGO, 
                            FCM_CATEGORIAAUDIT.NMCATEGORIAAUDIT AS NOME, 
                            RTRIM(XMLAGG(XMLELEMENT(E, FCM_RELACCATEGCRIT.NRCRITERIOAUDIT, ',').EXTRACT('//text()') ORDER BY FCM_RELACCATEGCRIT.NRCRITERIOAUDIT).GETCLOBVAL(), ',') AS CRITERIO,
                                NULL AS CODIGOPAI
                            FROM FCM_CATEGORIAAUDIT
                            JOIN FCM_RELACMODCATEG
                            ON FCM_RELACMODCATEG.NRCATEGORIAAUDIT = FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT
                            LEFT JOIN FCM_RELACCATEGCRIT
                            ON FCM_RELACCATEGCRIT.NRCATEGORIAAUDIT = FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT
                            WHERE FCM_RELACMODCATEG.NRMODELOAUDIT = :NRMODELOAUDIT
                            AND FCM_RELACCATEGCRIT.NRMODELOAUDIT = FCM_RELACMODCATEG.NRMODELOAUDIT
                            GROUP BY FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT, FCM_CATEGORIAAUDIT.NMCATEGORIAAUDIT

                    UNION ALL

                    SELECT 'T' || FCM_TOPICOAUDIT.NRTOPICOAUDIT AS CODIGO,
                            FCM_TOPICOAUDIT.NMTOPICOAUDIT AS NOME, 
                            NULL AS CRITERIO,
                            'C' || FCM_RELACCATTOPIC.NRCATEGORIAAUDIT AS CODIGOPAI
                            FROM FCM_TOPICOAUDIT
                            JOIN FCM_RELACCATTOPIC
                            ON FCM_RELACCATTOPIC.NRTOPICOAUDIT = FCM_TOPICOAUDIT.NRTOPICOAUDIT
                            JOIN FCM_RELACMODCATEG
                            ON FCM_RELACMODCATEG.NRCATEGORIAAUDIT = FCM_RELACCATTOPIC.NRCATEGORIAAUDIT
                            WHERE FCM_RELACMODCATEG.NRMODELOAUDIT = :NRMODELOAUDIT
                            AND FCM_RELACCATTOPIC.NRMODELOAUDIT = FCM_RELACMODCATEG.NRMODELOAUDIT
                            GROUP BY FCM_TOPICOAUDIT.NRTOPICOAUDIT, FCM_TOPICOAUDIT.NMTOPICOAUDIT, NULL, FCM_RELACCATTOPIC.NRCATEGORIAAUDIT";

        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros, [], null, null, null, array('CRITERIO'));
    }

    public function insertRelacaoCategoriaTopico($parametros)
    {
        $sql = "INSERT INTO FCM_RELACCATTOPIC (NRRELACCATTOPIC, NRCATEGORIAAUDIT, NRMODELOAUDIT,  NRTOPICOAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                    VALUES (:NRRELACCATTOPIC, :NRCATEGORIAAUDIT, :NRMODELOAUDIT, :NRTOPICOAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelacaoCategoriaCriterio($parametros)
    {
        $sql = "INSERT INTO FCM_RELACCATEGCRIT (NRRELACCATEGCRIT, NRCATEGORIAAUDIT, NRMODELOAUDIT, NRCRITERIOAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRRELACCATEGCRIT, :NRCATEGORIAAUDIT, :NRMODELOAUDIT, :NRCRITERIOAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";
    
        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelacaoModCategoria($parametros)
    {
        $sql = "INSERT INTO FCM_RELACMODCATEG (NRRELACMODCATEG, NRMODELOAUDIT, NRCATEGORIAAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
            VALUES (:NRRELACMODCATEG, :NRMODELOAUDIT, :NRCATEGORIAAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function validarTopicos($parametros)
    {
        $sql = "SELECT COUNT (*) AS CONT
                    FROM FCM_RELACCATTOPIC
                    WHERE NRCATEGORIAAUDIT =:NRCATEGORIAAUDIT
                    AND NRTOPICOAUDIT = :NRTOPICOAUDIT
                    AND NRMODELOAUDIT = :NRMODELOAUDIT";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0];
    }

    public function validarCategoria($parametros)
    {
        $sql = "SELECT COUNT (NRRELACMODCATEG) AS CONT
                    FROM FCM_RELACMODCATEG
                    WHERE NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT
                    AND NRMODELOAUDIT = :NRMODELOAUDIT";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0];
    }

    public function insertModeloAuditoria($parametros)
    {
        $sql = "INSERT INTO FCM_MODELOAUDIT (NRMODELOAUDIT, NMMODELOAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRMODELOAUDIT, :NMMODELOAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getModeloAuditoria()
    {
        $sql = "SELECT * FROM FCM_MODELOAUDIT
                ORDER BY NMMODELOAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Funções TAB Categorias
    public function getRelacaoCategorias($categorias)
    {
        $sql = "SELECT NRCATEGORIAAUDIT
                    FROM FCM_RELACMODCATEG
                    WHERE NRCATEGORIAAUDIT IN ($categorias)";
        
        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function insertCategoriaAuditoria($parametros)
    {
        $sql = "INSERT INTO FCM_CATEGORIAAUDIT (NRCATEGORIAAUDIT, NMCATEGORIAAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRCATEGORIAAUDIT, :NMCATEGORIAAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getCategoriaAuditoria()
    {
        $sql = "SELECT * FROM FCM_CATEGORIAAUDIT
                ORDER BY NMCATEGORIAAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteCategoria($categorias)
    {
        $sql = "DELETE FROM FCM_CATEGORIAAUDIT
                    WHERE NRCATEGORIAAUDIT IN ($categorias)";

        $this->connection->executeStatement($sql);
    }

    // Funções tab Tópicos
    public function getRelacaoTopicos($topicos)
    {
        $sql = "SELECT NRTOPICOAUDIT
                    FROM FCM_RELACCATTOPIC
                    WHERE NRTOPICOAUDIT IN ($topicos)";
        
        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function insertTopicosAuditoria($parametros)
    {
        $sql = "INSERT INTO FCM_TOPICOAUDIT (NRTOPICOAUDIT, NMTOPICOAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRTOPICOAUDIT, :NMTOPICOAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getTopicosAuditoria()
    {
        $sql = "SELECT * FROM FCM_TOPICOAUDIT
                ORDER BY NMTOPICOAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteTopicos($criterios)
    {
        $sql = "DELETE FROM FCM_TOPICOAUDIT
                    WHERE NRTOPICOAUDIT IN ($criterios)";

        $this->connection->executeStatement($sql);
    }

    // Funções da TAB de Critérios
    public function getRelacaoCriterios($criterios)
    {
        $sql = "SELECT NRCRITERIOAUDIT
                    FROM FCM_RELACCATEGCRIT
                    WHERE NRCRITERIOAUDIT IN ($criterios)";
        
        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function insertCriteriosAvaliacao($parametros)
    {
        $sql = "INSERT INTO FCM_CRITERIOAUDIT (NRCRITERIOAUDIT, NMCRITERIOAUDIT, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRCRITERIOAUDIT, :NMCRITERIOAUDIT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getCriteriosAvaliacao()
    {
        $sql = "SELECT * FROM FCM_CRITERIOAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteCriterios($criterios)
    {
        $sql = "DELETE FROM FCM_CRITERIOAUDIT
                    WHERE NRCRITERIOAUDIT IN ($criterios)";

        $this->connection->executeStatement($sql);
    }
}
