<?php

namespace FM\Service;

use FM\Factory\Fm00000000HabilidadesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Fm00000000HabilidadesService
{

    protected $habilidadeFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Fm00000000HabilidadesFactory $Fm00000000HabilidadesFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->habilidadeFactory              = $Fm00000000HabilidadesFactory;
        $this->Environment                    = $Environment;
        $this->newCode                        = $newCode;
        $this->connection                     = $entityManager->getConnection();
    }
    // FUNÇÕES DO FRAME PRINCIPAL HABILIDADES
    public function getHabilidades()
    {
        return $this->habilidadeFactory->getHabilidades();
    }

    public function salvarHabilidades($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            foreach ($params as $row) {
                if ($row["NRHABILIDADE"] == 0) {
                    $param = array(
                        "NRHABILIDADE"      => $this->newCode->getCodeByTableName("FCM_HABILIDADE", $nrorg, 1, 10, true),
                        "NMHABILIDADE"      => $row["NMHABILIDADE"],
                        "IDATIVO"           => 'S',
                        "NRORG"             => $nrorg,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERINCLUSAO"    => $operador
                    );

                    $this->habilidadeFactory->inserirHabilidade($param);
                } else {
                    $param = array(
                        "NRHABILIDADE"      => $row["NRHABILIDADE"],
                        "NMHABILIDADE"      => $row["NMHABILIDADE"],
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->habilidadeFactory->atualizarHabilidade($param);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirHabilidades($params)
    {
        $filtro = '';
        $atividadesEmUso = $this->habilidadeFactory->getAtividadesEmUso(implode(',', $params));

        if (empty($atividadesEmUso[0])) {
            $nr = implode(',', $params);
            $filtro .= "WHERE NRHABILIDADE IN (" . $nr . ")";
            $this->habilidadeFactory->excluirOperadorHabilidades($filtro);

            $this->excluirAtividadeHabilidades(array("DELHABI" => $params));

            $this->habilidadeFactory->excluirHabilidades($nr);
        } else {
            throw new \Exception("A habilidade não pôde ser excluida pois está vinculada a uma atividade com tarefa criada.");
        }
    }

    // FUNÇÕES TAB OEPRADORES
    public function getOperadorHabilidade($params)
    {
        $param = array(
            "NRHABILIDADE"    => $params["HABILIDADE"],
        );
        return $this->habilidadeFactory->getOperadorHabilidade($param);
    }

    public function relacionarOperadorHabilidade($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $param = array(
                "NRRELACHABOPER"    => $this->newCode->getCodeByTableName("FCM_RELACHABOPER", $nrorg, 1, 10, true),
                "NRHABILIDADE"      => $params["NRHABILIDADE"],
                "CDOPERADOR"        => $params["OPERADOR"],
                "NRORG"             => $nrorg,
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $operador,
                "IDATIVO"           => 'S',
            );

            $this->habilidadeFactory->relacionarOperadorHabilidade($param);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirOperadorHabilidades($params)
    {
        $param = implode(',', $params);
        $this->habilidadeFactory->excluirOperadorHabilidades($param);
    }

    // FUNÇÕES TAB ATIVIDADES
    public function getAtividadeHabilidade($params)
    {
        return $this->habilidadeFactory->getAtividadeHabilidade($params["HABILIDADE"]);
    }

    public function relacionarAtividadeHabilidade($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            //debugApi([$params]);
            $param = array(
                "NRRELACHABATIV"    => $this->newCode->getCodeByTableName("FCM_RELACHABATIV", $nrorg, 1, 10, true),
                "NRTAREFA"          => preg_replace("/[A-Za-z]/", "", $params["NRTAREFA"]),
                "NRHABILIDADE"      => $params["NRHABILIDADE"],
                "NRORG"             => $nrorg,
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $operador,
                "IDATIVO"           => 'S',
            );

            $this->habilidadeFactory->relacionarAtividadeHabilidade($param);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirAtividadeHabilidades($params)
    {
        $filtro = '';

        if (isset($params["NRROW"])) {
            $filtro .= "WHERE NRTAREFA IN (" . preg_replace("/[A-Za-z]/", "", $params["NRROW"]) . ")";
        } else if (isset($params["DELHABI"])) {
            $nrs = implode(',', $params["DELHABI"]);
            $filtro .= "WHERE NRHABILIDADE IN (" . $nrs . ")";
        } else {
            $nrs = implode(',', $params);
            $filtro .= "WHERE NRRELACHABATIV IN (" . $nrs . ")";
        }

        $this->habilidadeFactory->excluirAtividadeHabilidades($filtro);
    }
}
