import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, Form, LoadingService, Modal, ModalService, Select, Text, Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class DashboardTarefasController {
    private modal!: Modal;

    public currentRow: any;

    public visibilidadeGrid: boolean = true;

    public async criarModais() {
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00000_000_dash_tarefas/modal/fm_modal_transferir_tarefas' });
    }

    public destruirModais() {
        this.modal?.destroy();
    }

    public async abrirModal({ component, row }: any) {
        this.modal.show();
        this.visibilidadeGrid = (component.name === 'action-transferir');
        setTimeout(() => {
            this.validacoes(row);
        }, 500);
    }

    public validacoes(row: any) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        const operadorOrigem = Metadata.getInstance<Select>('operador-origem');

        operadorDestino.value = null;
        operadorOrigem.value = null;

        if (!this.visibilidadeGrid) {
            operadorOrigem.disabled = true;
            operadorDestino.disabled = false;
            operadorOrigem.value = row.CDOPERADOR;
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: row.NRHABILIDADE,
                CDOPERADOR: row.CDOPERADOR,
            });
            this.currentRow = row;
        } else {
            operadorOrigem.disabled = false;
            operadorDestino.disabled = false;
            this.defineTarefa();
        }
    }

    public defineTarefa() {
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        gridInterno.datasource.data = [];
        (gridInterno.datasource as MemoryDatasource).allData = [];

        gridPrincipal.selectedRows.forEach((element: any) => {
            const dados: any = {
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMOPERADOR: element.NMOPERADOR,
                CDOPERADOR: element.CDOPERADOR,
                DTHORAAGEND: element.DTHORAAGEND,
                NRHABILIDADE: element.NRHABILIDADE,
            };
            gridInterno.datasource.post(dados);
        });
        gridInterno.datasource.get();
    }

    public disableSelection({ row }: IDictionary) {
        const select = Metadata.getInstance<Select>('operador-origem');
        return row.CDOPERADOR !== select.value;
    }

    public changeOperadorDestino({ component }: IEventParam<Select>) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        Metadata.getInstance<TekGrid>('grid-tarefas-transferir').selectedRows = [];
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (component.value) {
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: component.selectValue.NRHABILIDADE,
                CDOPERADOR: component.value,
            });
        }

        if (this.visibilidadeGrid) {
            gridInterno.datasource.setFilter({
                CDOPERADOR: component.value,
            });
        }
    }

    public transferirTarefa() {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino').value;
        const operadorOrigem = Metadata.getInstance<Select>('operador-origem').value;
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (Utilities.validateForm(Metadata.getInstance<Form>('form-transferencia'))) {
            if (operadorOrigem !== operadorDestino) {
                if (this.visibilidadeGrid) {
                    if (gridInterno.selectedRows.length > 0) {
                        this.salvarTransferencia(operadorOrigem, operadorDestino, gridInterno.selectedRows).then(async () => {
                            gridPrincipal.datasource.get();
                            await gridInterno.deleteRows();
                            gridPrincipal.selectedRows = [];
                        }).finally(() => {
                            LoadingService.hide();
                        });
                    } else {
                        Utilities.alertMessage('Selecione pelo menos 1 tarefa para transferir', 'warning');
                    }
                }

                if (!this.visibilidadeGrid) {
                    this.salvarTransferencia(operadorOrigem, operadorDestino, this.currentRow).then(() => {
                        gridPrincipal.datasource.get();
                        this.fecharModal();
                    }).finally(() => {
                        LoadingService.hide();
                    });
                }
            } else {
                Utilities.alertMessage('Selecione um Operador de Destino diferente do Operador de Origem', 'warning');
            }
        }
    }

    public async salvarTransferencia(origem: any, destino: any, tarefa: any) {
        return Http.post('/transferirTarefas', {
            OPERADORORIGEM: origem,
            OPERADORDESTINO: destino,
            TAREFAS: tarefa,
        }).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public fecharModal() {
        this.modal.hide();
    }

    public setFiltro({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    public montarInformacoesComplementares() {
        LoadingService.show();
        const nr = this.currentRow.NRCHECKLISTAGEND;
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens');
        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        let mensagem = '';

        Http.post('/getItensTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: IDictionary) => {
            const dados = response.data.data;

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (this.currentRow.DSJUSTCANCELAMENTO !== null && this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO} - Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else if (this.currentRow.DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO}`;
            } else if (this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else {
                mensagem = 'Não há observações';
            }
            observacoes.value = mensagem;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos({ component }: any) {
        const nr = this.currentRow.NRCHECKLISTAGEND;

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public disabledAction: boolean = false;

    public rowStyleConditions(row: any) {
        if (row.ATRASO.split(' ')[1] === 'dias') {
            if (row.ATRASO.split(' ')[0] >= 1) {
                return { cssClass: 'linha-grid-verificada' };
            }
        }
        return { cssClass: '' };
    }

    private clicks: any = 0;

    // função que simula um double click
    public rowClick({ row }: any) {
        this.currentRow = row;
        this.clicks += 1;
        if (this.clicks >= 2) {
            // this.abrirModal();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        }, 500);
    }
}
