import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    Form, LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekTreeGrid } from '@zeedhi/teknisa-components-common';

export class Fm00003003AtividadeCriterioController {
    private modalAdicionarAtvCriterio!: Modal;

    private currentRow: any = '';

    public tituloModal: String = 'Adicionar Atividade ou Critério';

    private clicks = 0;

    private parametrizacao: String = '';

    // função que simula um double click
    public rowClick({ row }: any) {
        this.clicks += 1;
        this.currentRow = row;

        if (this.clicks >= 2) {
            this.abrirModalEdicao();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        },
            500);
    }

    public async abrirModal() {
        this.modalAdicionarAtvCriterio = await ModalService.createFromJsonPost('', { jsonPath: '/fm_00003_000_checklist/modal/fm_00003_003_salvar_atividade_criterio' });
        await this.modalAdicionarAtvCriterio.show();
    }

    public async abrirModalInserir() {
        await this.abrirModal();
        this.tituloModal = 'Adicionar Atividade ou Critério';

        if (this.currentRow !== '') {
            if (this.currentRow.TIPO === 'Atividade') {
                Metadata.getInstance<Select>('TIPOADICAO').value = 'Critério';
                Metadata.getInstance<Select>('ATIVIDADES').value = this.currentRow.NR;
            }
        }
    }

    public async getParametrizacoes() {
        Http.get('/getParametrizacoes').then((response) => {
            if (response.data.length > 0) {
                this.parametrizacao = response.data[0].IDUTILIZAHABILIDADE;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public async abrirModalEdicao() {
        await this.abrirModal();
        const nr = Metadata.getInstance<TextInput>('NRROW');
        const tipoAdicao = Metadata.getInstance<Select>('TIPOADICAO');
        this.tituloModal = 'Editar Atividade ou Critério';

        if (this.currentRow !== '') {
            nr.value = this.currentRow.CODIGO;
            tipoAdicao.value = this.currentRow.TIPO;
            tipoAdicao.disabled = true;
            Metadata.getInstance<TextInput>('NOME').value = this.currentRow.NOME;
            const compHabilidade = Metadata.getInstance<SelectMultiple>('HABILIDADES');
            if (this.parametrizacao === 'S') {
                compHabilidade.disabled = false;
                compHabilidade.setValue(this.currentRow.HABILIDADE.map((value: any) => parseInt(value, 10)));
            } else {
                compHabilidade.disabled = true;
                compHabilidade.clearValidations();
            }
        }
    }

    public fecharModal() {
        this.modalAdicionarAtvCriterio.hide();
        this.modalAdicionarAtvCriterio?.destroy();
    }

    public habilitarCampoAtividade({ component }: any) {
        const campoAtividade = Metadata.getInstance<Select>('ATIVIDADES');

        if (component.value === 'Critério') {
            campoAtividade.isVisible = true;
            campoAtividade.addValidation('required', {});
        } else {
            campoAtividade.isVisible = false;
            campoAtividade.clearValidations();
        }
    }

    public salvarAtividadeCriterio() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-atividades-criterios');
        const form = Metadata.getInstance<Form>('form-ac');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/salvarAtividadeCriterio', {
                NRROW: form.value.NRROW,
                NOME: form.value.NOME,
                ATIVIDADES: form.value.ATIVIDADES,
                TIPOADICAO: form.value.TIPOADICAO,
                HABILIDADES: form.value.HABILIDADES,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                this.fecharModal();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public dialogInativarAtividadesCriterios({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003003AtividadeCriterioController.inativarAtividadesCriterios', 'Inativar Registro', 'Deseja realmente inativar todos os registros selecionados?');
    }

    public inativarAtividadesCriterios() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-atividades-criterios');

        Utilities.deleteInfoGrid('/inativarAtividadesCriterios', { component: grid }).then((response: any) => {
            DialogService.hide();
            if (response.data === 'err') {
                Utilities.showDialog('Atenção!', 'warning', 'Alguns registros não puderam ser inativados pois possuem filhos ativos!');
            }
        });
    }
}
