/* eslint-disable */
import {
    Http, IDictionary, Loader, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { CheckboxMultiple, Loading, LoadingService, Modal, ModalService, Text, Textarea } from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class RelatorioTarefasController {
    private modal!: Modal;

    public currentRow: any;

    public async abrirModal() {
        LoadingService.show();
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00028_000_relatorios/modal/visualizar_infos_relatorio_tarefas' });
        this.modal.show();
        LoadingService.hide();
    }

    public closeModal() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public setFiltro({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    public montarInformacoesComplementares() {
        LoadingService.show();
        const nr = this.currentRow.NRCHECKLISTAGEND;
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens');
        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        let mensagem = ''

        Http.post('/getItensTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: IDictionary) => {
            const dados = response.data.data;

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (this.currentRow.DSJUSTCANCELAMENTO !== null && this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO} - Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else if (this.currentRow.DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO}`;
            } else if (this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else {
                mensagem = "Não há observações";
            }
            observacoes.value = mensagem;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos({ component }: any) {
        const nr = this.currentRow.NRCHECKLISTAGEND;

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: nr
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public rowStyleConditions(row: any) {
        if (row.ATRASO.split(' ')[1] === 'dias') {
            if (row.ATRASO.split(' ')[0] >= 1) {
                return { cssClass: 'linha-grid-verificada' };
            }
        }
        return { cssClass: '' };
    }

    private clicks: any = 0;

    // função que simula um double click
    public rowClick({ row }: any) {
        this.currentRow = row;
        this.clicks += 1;
        if (this.clicks >= 2) {
            this.abrirModal();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        }, 500);
    }
}
